@@ -1,5 +1,5 @@
 use zellij_utils::pane_size::Size;
-use zellij_utils::{interprocess, libc, nix, signal_hook, zellij_tile};
+use zellij_utils::{interprocess, libc, nix, signal_hook};
 
 use interprocess::local_socket::LocalSocketStream;
 use mio::{unix::SourceFd, Events, Interest, Poll, Token};
@@ -11,8 +11,8 @@ use std::os::unix::io::RawFd;
 use std::path::Path;
 use std::sync::{Arc, Mutex};
 use std::{io, thread, time};
-use zellij_tile::data::Palette;
 use zellij_utils::{
+    data::Palette,
     errors::ErrorContext,
     ipc::{ClientToServerMsg, IpcReceiverWithContext, IpcSenderWithContext, ServerToClientMsg},
     shared::default_palette,
@@ -27,7 +27,7 @@ fn into_raw_mode(pid: RawFd) {
     let mut tio = termios::tcgetattr(pid).expect("could not get terminal attribute");
     termios::cfmakeraw(&mut tio);
     match termios::tcsetattr(pid, termios::SetArg::TCSANOW, &tio) {
-        Ok(_) => {}
+        Ok(_) => {},
         Err(e) => panic!("error {:?}", e),
     };
 }
@@ -55,10 +55,21 @@ pub(crate) fn get_terminal_size_using_fd(fd: RawFd) -> Size {
     unsafe {
         ioctl(fd, TIOCGWINSZ.into(), &mut winsize)
     };
-    Size {
-        rows: winsize.ws_row as usize,
-        cols: winsize.ws_col as usize,
-    }
+
+    // fallback to default values when rows/cols == 0: https://github.com/zellij-org/zellij/issues/1551
+    let rows = if winsize.ws_row != 0 {
+        winsize.ws_row as usize
+    } else {
+        24
+    };
+
+    let cols = if winsize.ws_col != 0 {
+        winsize.ws_col as usize
+    } else {
+        80
+    };
+
+    Size { rows, cols }
 }
 
 #[derive(Clone)]
@@ -83,7 +94,7 @@ pub trait ClientOsApi: Send + Sync {
     fn get_stdout_writer(&self) -> Box<dyn io::Write>;
     fn get_stdin_reader(&self) -> Box<dyn io::Read>;
     /// Returns the raw contents of standard input.
-    fn read_from_stdin(&self) -> Vec<u8>;
+    fn read_from_stdin(&mut self) -> Vec<u8>;
     /// Returns a [`Box`] pointer to this [`ClientOsApi`] struct.
     fn box_clone(&self) -> Box<dyn ClientOsApi>;
     /// Sends a message to the server.
@@ -115,7 +126,7 @@ impl ClientOsApi for ClientOsInputOutput {
     fn box_clone(&self) -> Box<dyn ClientOsApi> {
         Box::new((*self).clone())
     }
-    fn read_from_stdin(&self) -> Vec<u8> {
+    fn read_from_stdin(&mut self) -> Vec<u8> {
         let stdin = std::io::stdin();
         let mut stdin = stdin.lock();
         let buffer = stdin.fill_buf().unwrap();
@@ -161,11 +172,11 @@ impl ClientOsApi for ClientOsInputOutput {
                     }
                     sigwinch_cb_timestamp = time::Instant::now();
                     sigwinch_cb();
-                }
+                },
                 SIGTERM | SIGINT | SIGQUIT | SIGHUP => {
                     quit_cb();
                     break;
-                }
+                },
                 _ => unreachable!(),
             }
         }
@@ -177,10 +188,10 @@ impl ClientOsApi for ClientOsInputOutput {
                 Ok(sock) => {
                     socket = sock;
                     break;
-                }
+                },
                 Err(_) => {
                     std::thread::sleep(std::time::Duration::from_millis(50));
-                }
+                },
             }
         }
         let sender = IpcSenderWithContext::new(socket);
