@@ -123,8 +123,13 @@ pub enum ScreenInstruction {
     ChangeMode(ModeInfo, ClientId),
     LeftClick(Position, ClientId),
     RightClick(Position, ClientId),
-    MouseRelease(Position, ClientId),
-    MouseHold(Position, ClientId),
+    MiddleClick(Position, ClientId),
+    LeftMouseRelease(Position, ClientId),
+    RightMouseRelease(Position, ClientId),
+    MiddleMouseRelease(Position, ClientId),
+    MouseHoldLeft(Position, ClientId),
+    MouseHoldRight(Position, ClientId),
+    MouseHoldMiddle(Position, ClientId),
     Copy(ClientId),
     AddClient(ClientId),
     RemoveClient(ClientId),
@@ -222,8 +227,13 @@ impl From<&ScreenInstruction> for ScreenContext {
             ScreenInstruction::ScrollDownAt(..) => ScreenContext::ScrollDownAt,
             ScreenInstruction::LeftClick(..) => ScreenContext::LeftClick,
             ScreenInstruction::RightClick(..) => ScreenContext::RightClick,
-            ScreenInstruction::MouseRelease(..) => ScreenContext::MouseRelease,
-            ScreenInstruction::MouseHold(..) => ScreenContext::MouseHold,
+            ScreenInstruction::MiddleClick(..) => ScreenContext::MiddleClick,
+            ScreenInstruction::LeftMouseRelease(..) => ScreenContext::LeftMouseRelease,
+            ScreenInstruction::RightMouseRelease(..) => ScreenContext::RightMouseRelease,
+            ScreenInstruction::MiddleMouseRelease(..) => ScreenContext::MiddleMouseRelease,
+            ScreenInstruction::MouseHoldLeft(..) => ScreenContext::MouseHoldLeft,
+            ScreenInstruction::MouseHoldRight(..) => ScreenContext::MouseHoldRight,
+            ScreenInstruction::MouseHoldMiddle(..) => ScreenContext::MouseHoldMiddle,
             ScreenInstruction::Copy(..) => ScreenContext::Copy,
             ScreenInstruction::ToggleTab(..) => ScreenContext::ToggleTab,
             ScreenInstruction::AddClient(..) => ScreenContext::AddClient,
@@ -641,11 +651,6 @@ impl Screen {
     pub fn new_tab(&mut self, layout: Layout, new_pids: Vec<RawFd>, client_id: ClientId) {
         let tab_index = self.get_new_tab_index();
         let position = self.tabs.len();
-        let client_mode_info = self
-            .mode_info
-            .get(&client_id)
-            .unwrap_or(&self.default_mode_info)
-            .clone();
         let mut tab = Tab::new(
             tab_index,
             position,
@@ -657,7 +662,7 @@ impl Screen {
             self.bus.senders.clone(),
             self.max_panes,
             self.style,
-            client_mode_info,
+            self.default_mode_info.clone(),
             self.draw_pane_frames,
             self.connected_clients.clone(),
             self.session_is_mirrored,
@@ -702,21 +707,29 @@ impl Screen {
     }
 
     pub fn add_client(&mut self, client_id: ClientId) {
-        let mut tab_index = 0;
         let mut tab_history = vec![];
-        if let Some((_first_client, first_active_tab_index)) = self.active_tab_indices.iter().next()
-        {
-            tab_index = *first_active_tab_index;
-        }
         if let Some((_first_client, first_tab_history)) = self.tab_history.iter().next() {
             tab_history = first_tab_history.clone();
         }
+
+        let tab_index = if let Some((_first_client, first_active_tab_index)) =
+            self.active_tab_indices.iter().next()
+        {
+            *first_active_tab_index
+        } else if self.tabs.contains_key(&0) {
+            0
+        } else if let Some(tab_index) = self.tabs.keys().next() {
+            tab_index.to_owned()
+        } else {
+            panic!("Can't find a valid tab to attach client to!");
+        };
+
         self.active_tab_indices.insert(client_id, tab_index);
         self.connected_clients.borrow_mut().insert(client_id);
         self.tab_history.insert(client_id, tab_history);
         self.tabs
             .get_mut(&tab_index)
-            .unwrap()
+            .unwrap_or_else(|| panic!("Failed to attach client to tab with index {tab_index}"))
             .add_client(client_id, None);
     }
     pub fn remove_client(&mut self, client_id: ClientId) {
@@ -1300,14 +1313,42 @@ pub(crate) fn screen_thread_main(
                 screen.update_tabs();
                 screen.render();
             },
-            ScreenInstruction::MouseRelease(point, client_id) => {
+            ScreenInstruction::MiddleClick(point, client_id) => {
                 active_tab!(screen, client_id, |tab: &mut Tab| tab
-                    .handle_mouse_release(&point, client_id));
+                    .handle_middle_click(&point, client_id));
+                screen.update_tabs();
                 screen.render();
             },
-            ScreenInstruction::MouseHold(point, client_id) => {
+            ScreenInstruction::LeftMouseRelease(point, client_id) => {
+                active_tab!(screen, client_id, |tab: &mut Tab| tab
+                    .handle_left_mouse_release(&point, client_id));
+                screen.render();
+            },
+            ScreenInstruction::RightMouseRelease(point, client_id) => {
+                active_tab!(screen, client_id, |tab: &mut Tab| tab
+                    .handle_right_mouse_release(&point, client_id));
+                screen.render();
+            },
+            ScreenInstruction::MiddleMouseRelease(point, client_id) => {
+                active_tab!(screen, client_id, |tab: &mut Tab| tab
+                    .handle_middle_mouse_release(&point, client_id));
+                screen.render();
+            },
+            ScreenInstruction::MouseHoldLeft(point, client_id) => {
                 active_tab!(screen, client_id, |tab: &mut Tab| {
-                    tab.handle_mouse_hold(&point, client_id);
+                    tab.handle_mouse_hold_left(&point, client_id);
+                });
+                screen.render();
+            },
+            ScreenInstruction::MouseHoldRight(point, client_id) => {
+                active_tab!(screen, client_id, |tab: &mut Tab| {
+                    tab.handle_mouse_hold_right(&point, client_id);
+                });
+                screen.render();
+            },
+            ScreenInstruction::MouseHoldMiddle(point, client_id) => {
+                active_tab!(screen, client_id, |tab: &mut Tab| {
+                    tab.handle_mouse_hold_middle(&point, client_id);
                 });
                 screen.render();
             },
