@@ -5,9 +5,8 @@ use crate::ui::boundaries::Boundaries;
 use crate::ui::pane_boundaries_frame::FrameParams;
 use crate::ClientId;
 use std::collections::HashMap;
-use zellij_tile::{
-    data::{client_id_to_colors, single_client_color, InputMode, PaletteColor},
-    prelude::Style,
+use zellij_utils::data::{
+    client_id_to_colors, single_client_color, InputMode, PaletteColor, Style,
 };
 pub struct PaneContentsAndUi<'a> {
     pane: &'a mut Box<dyn Pane>,
@@ -45,13 +44,19 @@ impl<'a> PaneContentsAndUi<'a> {
         &mut self,
         clients: impl Iterator<Item = ClientId>,
     ) {
-        if let Some((character_chunks, raw_vte_output)) = self.pane.render(None) {
+        if let Some((character_chunks, raw_vte_output, sixel_image_chunks)) = self.pane.render(None)
+        {
             let clients: Vec<ClientId> = clients.collect();
             self.output.add_character_chunks_to_multiple_clients(
                 character_chunks,
                 clients.iter().copied(),
                 self.z_index,
             );
+            self.output.add_sixel_image_chunks_to_multiple_clients(
+                sixel_image_chunks,
+                clients.iter().copied(),
+                self.z_index,
+            );
             if let Some(raw_vte_output) = raw_vte_output {
                 self.output.add_post_vte_instruction_to_multiple_clients(
                     clients.iter().copied(),
@@ -66,9 +71,16 @@ impl<'a> PaneContentsAndUi<'a> {
         }
     }
     pub fn render_pane_contents_for_client(&mut self, client_id: ClientId) {
-        if let Some((character_chunks, raw_vte_output)) = self.pane.render(Some(client_id)) {
+        if let Some((character_chunks, raw_vte_output, sixel_image_chunks)) =
+            self.pane.render(Some(client_id))
+        {
             self.output
                 .add_character_chunks_to_client(client_id, character_chunks, self.z_index);
+            self.output.add_sixel_image_chunks_to_client(
+                client_id,
+                sixel_image_chunks,
+                self.z_index,
+            );
             if let Some(raw_vte_output) = raw_vte_output {
                 self.output.add_post_vte_instruction_to_client(
                     client_id,
@@ -202,7 +214,7 @@ impl<'a> PaneContentsAndUi<'a> {
                         let colors = client_id_to_colors(client_id, self.style.colors);
                         colors.map(|colors| colors.0)
                     }
-                }
+                },
                 _ => Some(self.style.colors.orange),
             }
         } else {
